package edu.dhbw.pv;

public class Matrix {

    double[][] val;
    
    public Matrix(double[][] v) {
        val = v;
    }
    
    public Matrix(int n, int m) {
        this( new double[n][m] );
    }
    
    public static Matrix one(int n, int m) {
        double[][] v = new double[n][m];
        for (int i = 0; i < Math.min(n, m); i++) {
            v[i][i] = 1.0;
        }
        return new Matrix(v);
    }
    
    public static Matrix random(int n, int m) {
        double[][] v = new double[n][m];
        for (int i = 0; i < n; i++ ) {
            for (int j= 0; j < m; j++) {
                v[i][j] = Math.random()*10.0;
            }
        }
        return new Matrix(v);
    }
    
    public Matrix transpose() {
        double[][] v = new double[val[0].length][val.length];
        for (int i = 0; i < val.length; i++ ) {
            for (int j = 0; j < val[0].length; j++)
                 v[j][i] = val[i][j];
        }
        return new Matrix(v);
    }
    
    double epsilon = 1.0e-10;
    public boolean equals(Object b) {
        if (! (b instanceof Matrix) ) {
            return false;
        }
        Matrix B = (Matrix)b;
        for (int i = 0; i < val.length; i++) {
            for (int j = 0; j < val[0].length; j++ ) {
                if ( Math.abs(val[i][j] - B.val[i][j]) > epsilon ) {
                    return false;
                }
            }
        }
        return true;
    }
    
    public String toString() {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < val.length; i++) {
            for (int j = 0; j < val[0].length; j++) {
                sb.append( val[i][j] + " ");
            }
            sb.append("\n");
        }
        return sb.toString();
    }
    
    public Matrix multiply(Matrix B) {
        double[][] erg = new double[val.length][B.val[0].length];
        if (val[0].length != B.val.length) {
            throw new RuntimeException("Matrix missmatch");
        }
        for (int i = 0; i < erg.length; i++) {
            for (int j = 0; j < erg[0].length; j++ ) {
                double s = 0.0;
                for (int k = 0; k < val[0].length; k++ ) {
                    s += val[i][k] * B.val[k][j];
                }
                erg[i][j] = s;
            }
        }
        return new Matrix(erg);
    }
    
}
