use structs::Matrix;
use traits::ParallelMatrixMult;
use std::sync::mpsc;

extern crate crossbeam;


impl ParallelMatrixMult for Matrix{
    fn parallel_mult(&self,b:&Matrix,num_threads:i32) -> Matrix{
        let rows = self.get_rows();
        let cols = b.get_cols();
        let mut c = Matrix::new(rows,cols);
        let mut thread_com = Vec::with_capacity(8);
        let (t2mtx,t2mrx) = mpsc::channel();
        crossbeam::scope(|scope|{
            for _ in 0..num_threads{
                let (m2ttx,rx) = mpsc::channel();
                thread_com.push(m2ttx);
                let tx = t2mtx.clone();

                scope.spawn(move || {
                    for r in rx {
                        self.row_mult(b,r,|r,j,sum| {tx.send((r, j, sum)).unwrap()});
                    }
                });

            }
            for i in 0..rows{
                let sender =thread_com.remove(0);
                sender.send(i).unwrap();
                thread_com.push(sender);
            }
            drop(t2mtx);
            drop(thread_com);
            for recv in t2mrx{
                let (p1,p2,p3) = recv;
                c.set(p1,p2,p3);
            }
        });

        c
    }
    fn parallel_mult_row_threads(&self,b:&Matrix) -> Matrix{
        let rows = self.get_rows();
        let cols = b.get_cols();
        let mut c = Matrix::new(rows,cols);
        let (t2mtx,t2mrx) = mpsc::channel();
        crossbeam::scope(|scope|{
            for i in 0..rows{
                let tx = t2mtx.clone();
                scope.spawn(move || {
                    self.row_mult(b,i,|r,j,sum| {tx.send((r, j, sum)).unwrap()});
                });
            }
        });
        drop(t2mtx);
        for rec in t2mrx{
            let (p1,p2,p3) = rec;
            c.set(p1,p2,p3);
        }
        c
    }

    fn parallel_transpose_mult(&self,b:&Matrix,num_threads:i32) -> Matrix{
        let rows = self.get_rows();
        let cols = b.get_cols();
        let mut c = Matrix::new(rows,cols);
        let b = &b.transpose();
        let mut thread_com = Vec::with_capacity(8);
        let (t2mtx,t2mrx) = mpsc::channel();
        crossbeam::scope(|scope|{
            for _ in 0..num_threads{
                let (m2ttx,rx) = mpsc::channel();
                thread_com.push(m2ttx);
                let tx = t2mtx.clone();

                scope.spawn(move || {
                    for r in rx {
                        self.row_mult_transposed(b,r,|r,j,sum| {tx.send((r, j, sum)).unwrap()});
                    }
                });

            }
            for i in 0..rows{
                let sender =thread_com.remove(0);
                sender.send(i).unwrap();
                thread_com.push(sender);
            }
            drop(t2mtx);
            drop(thread_com);
            for recv in t2mrx{
                let (p1,p2,p3) = recv;
                c.set(p1,p2,p3);
            }
        });

        c
    }
}