use traits::Print;
use std::cmp::PartialEq;
use std::fmt;


extern crate time;
extern crate rand;

pub struct Matrix {
    cols: i32,
    rows: i32,
    data: Vec<f64>
}

impl Matrix {
    pub fn new(cols: i32, rows: i32) -> Matrix {
        let mut data = vec![5.0; cols as usize * rows as usize];
        for i in 0..data.len() {
            data[i] = rand::random::<f64>();
        }
        Matrix {
            cols: cols,
            rows: rows,
            data: data
        }
    }

    pub fn get(&self,col: i32, row: i32) -> f64{
        self.data[(self.cols*row+col) as usize]
    }

    pub fn set(& mut self, col:i32,row:i32,data:f64){
        self.data[(self.cols*row+col) as usize] = data;
    }

    pub fn get_cols(&self) -> i32{
        self.cols
    }

    pub fn get_rows(&self) -> i32{
        self.rows
    }

    pub fn get_row_data(&self,row:i32) -> Vec<f64>{
        self.data[(row*self.cols)as usize..((row+1)*self.cols)as usize].to_vec()
    }

    pub fn get_data(&self) -> &Vec<f64>{
        &self.data
    }

    pub fn row_mult_transposed<F>(&self, b:&Matrix, row:i32,func:F) where F : Fn(i32,i32,f64)
    {
        for j in 0..b.get_cols() {
            let mut sum = 0 as f64;
            for k in 0..b.get_rows() {
                sum += self.get(row, k) * b.get(k, j);
            }
            func(row,j,sum);
        }
    }

    pub fn row_mult<F>(&self,b:&Matrix,row:i32,func:F) where F : Fn(i32,i32,f64){
        for j in 0..b.get_cols() {
            let mut sum = 0 as f64;
            for k in 0..b.get_rows() {
                sum += self.get(row, k) * b.get(j, k);
            }
            func(row,j,sum);
        }
    }

    pub fn transpose(&self) -> Matrix{
        let start = time::precise_time_ns();
        let mut b = Matrix::new(self.cols,self.rows);
        for i in 0..self.cols{
            for j in 0..self.rows{
                b.set(i,j,self.get(j,i));
            }
        }
        let end = time::precise_time_ns();
        println!("Transpose took: {} ms",(end-start)/1e6 as u64 );
        b
    }
}

impl Print for Matrix{
    fn print(&self){
        for i in 0..(self.get_rows()){
            println!("{:?}",self.get_row_data(i));
        }
    }
}

impl Clone for Matrix{
    fn clone(&self) -> Self{
        let mut m = Matrix::new(self.cols,self.rows);
        m.data = self.data.clone();
        m
    }
}

impl fmt::Debug for Matrix{
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        try!(write!(f, "["));
        write!(f, "]")
    }
}

impl PartialEq for Matrix{
    fn eq(&self, other: &Matrix) -> bool{
        let mut test = true;
        for i in 0..self.data.len(){
            if !(self.data[i] == other.get_data()[i]) {
                test = false;
            }
        }
        self.cols ==  other.get_cols() &&
        self.rows == other.get_rows() &&
        test
    }
}