#include "ParMultProcTrans.h"

#include "unistd.h"

#include <iostream>
#include <algorithm>

using namespace std;

/**
 * Parallel Matrix Multiplication. 
 * Using specified number of threads, matrix B is transposed.
 * @author Joachim Staats.
 */

/**
 * Performs the multiplication of two matrices, B transposed.
 * C = A * transpose(B).
 * @param C result matrix.
 * @param A matrix.
 * @param B matrix.
 */
void ParMultProcTrans::multiply(double* C, double* A, double* B, int m, int p, int n) 
{
    pthread_t t[anzahl];
    eight_args params[m];
    int thread_present[m];
    int res;
    cout << "Starting " << anzahl << " threads ...";
    for (int i=0; i < anzahl; i++) {
        thread_present[i] = 1;
        params[i].C = C;
        params[i].A = A;
        params[i].B = B;
        params[i].m = m;
        params[i].p = p;
        params[i].n = n;
        params[i].i = i; 
        params[i].anzahl = anzahl;
        res = pthread_create(&t[i], NULL, RowMultProcTrans,(void *)&params[i]);
        if (res != 0)
        {
            thread_present[i] = 0;
            RowMultProcTrans(&params[i]);
        }
    }
    cout << " started ...";
    for (int i=0; i < anzahl; i++) {
        if (thread_present[i] != 0)
            pthread_join (t[i], NULL);
    }
    cout << " done ParMultProcTrans ";
}


/**
 * This function performs a row multiplication.
 */
void* RowMultProcTrans (void *arg) 
{
    eight_args *params = (eight_args *)arg;
    double* C = params->C;
    double* A = params->A;
    double* B = params->B;
    int m = params->m;
    int p = params->p;
    int n = params->n;
    int i = params->i;
    int anzahl = params->anzahl;

    int schritte = m / anzahl; 
    if ( (m % anzahl) != 0 ) {
        schritte++; // ceiling
    }
    for (int ii = i*schritte; ii < min((i+1)*schritte,m); ii++ ) {
        double* Aii = &A[ii];
        for (int j=0; j < p; j++) {
            double* Bj = &B[j];
            double c = 0.0;
            for (int k=0; k < n; k++) {
                 c += Aii[k] * Bj[k];
            }
            C[ii*p+j] = c;
        }
    }
}
