
/**
 * Konto version B.
 * Uses synchronized methods to display, add and remove money 
 * from the account, account cannot become negative.
 * @author Heinz Kredel.
 */ 
public class KontoB implements Konto {

    private double stand; // Konto Stand

    public KontoB() { stand = 0.0; } // sets Kontostand to desired value

    public KontoB(double a) { stand = a; }
    
/**
 * Displays account balance.
 * @return stand.
 */
        public synchronized double zeigeGeld() { return stand; }

/**
 * Transfers amount to this account.
 * Blocks if account would become negative.
 * @param b amount to put on account.
 * @return s returns actual balance from account.
 */
    public synchronized double putGeld(double b) { // stand += b
        double s = stand + b;
        WorkB.schaffen(100);
        // nur falls s >= 0
        while (s < 0 && stand > s) {
                try { 
                    this.wait();       
                } catch(InterruptedException e) { } 
                s = stand + b;
        }
        if (s >= 0 || s > stand) this.notify();
        stand = s;
        return s;   
    }

    
/**
 * New account balance gets calculated and returned.
 * Blocks if account would become negative.
 * @param b amount to remove from desired account.
 * @return s current account balance.
 */
    public synchronized double getGeld(double b) { // stand -= b
        double s = stand - b;
        WorkB.schaffen(100);
        // nur falls s >= 0
        while (s < 0 && stand > s) {
                try { 
                    this.wait();       
                } catch(InterruptedException e) { } 
                s = stand - b;
        }
        if (s >= 0 || s > stand) this.notify();
        stand = s;
        return s;   
    }

}


/**
 * sending current thread to sleep for a random time.
 */
class WorkB {

/**
 * @param max defines range of numbers for Math.random().
 */
    public static void schaffen(int max) {
        try {
            Thread.currentThread().sleep( (int)(max*Math.random()) );
        } catch (InterruptedException e) {
              System.out.println("Dies sollte nicht passieren "+e);
        }
    }
}
