import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import mpp.*;

/**
 * Computes Pi. To compile and run:
 *  javac -classpath mpp.jar Pi.java
 *  java -classpath mpp.jar:. Pi
 */
public class Pi {

  public static void main(String[] args) throws IOException {
    Communicator comm = new BlockCommunicator();
    BufferedReader read =
      new BufferedReader(new InputStreamReader(System.in));

    int size = comm.size(), rank = comm.rank();

    while (true) {
      int[] n = new int[1];
      if (rank == 0) {
        System.out.print("Enter the number of intervals: (0 quits) ");
        n[0] = Integer.parseInt((read.readLine()+" 0").trim().split(" ")[0]);
      }

      long startTime = System.currentTimeMillis();
      comm.bcast(n, 0);

      if (n[0] == 0)
        break;

      double[] pi = compPi(size, rank, n[0]);
      double[] mypi = new double[1];
      comm.reduce(pi, mypi, Reductions.sum(), 0);

      long endTime = System.currentTimeMillis() - startTime;
      if (rank == 0) {
        System.out.println(
          "pi is approximately "
            + mypi[0]
            + ", error is "
            + Math.abs(mypi[0] - Math.PI));
        System.out.println("time is " + endTime / 1000. + " seconds");
      }
    }

    comm.close();
  }

  private static double[] compPi(int size, int rank, int n) {
    double h = 1. / n;
    double sum = 0.;
    for (int i = rank + 1; i < n; i += size) {
      double x = h * (i - 0.5);
      sum += (4. / (1. + x * x));
    }

    return new double[] { h * sum };
  }
}
