
package algo;

import java.io.Serializable;

/**
 * A Graph for an TSP Problem.
 * @author Heinz Kredel.
 */
public class Graph implements Serializable {

    protected double[][] g;
    final static double INFINITY = Double.POSITIVE_INFINITY;
                              // = unconnected

/**
 * Constructs an unconnected graph of given size.
 * @param n size of the graph.
 */
    public Graph(int n) {
        g = new double[n][n];
        for (int i = 0; i < n; i++ ) {
            connect(i,i,0.0);
            for (int j = i+1; j < n; j++ ) {
                connect(i,j,INFINITY);
                connect(j,i,INFINITY);
            }
        }
    }


/**
 * Connect two nodes by defining a distance.
 * @param i node number.
 * @param j node number.
 * @param d distance between the two nodes.
 */
    public void connect(int i, int j, double d) {
        g[i][j] = d;
    }


/**
 * Get the distance between two nodes.
 * @param i node number.
 * @param j node number.
 * @return distance between the two nodes.
 */
    public double distance(int i, int j) {
        return g[i][j];
    }


/**
 * @return size of the graph.
 */
    public int size() {
        return g.length;
    }

}