
/**
 * A Path in a Graph implementd as Arrays of bytes. 
 */

public final class PathByteArray implements Path {

    protected final Graph graph;
    protected final byte[] used;   // points in the path
    protected final byte[] unused; // points not in the path
    public final double cost;
    public final int maxsize;
    public final int length;

    public PathByteArray(Graph g, byte[] u, byte[] uu, double c) {
        graph = g;
        used = u;
        unused = uu;
        cost = c;
        length = used.length;
        maxsize = graph.size();
    }


    public double cost() {
        return cost;
    }

    public int length() {
        return length;
    }

    public int maxsize() {
        return maxsize;
    }


    /**
     * empty path, all n points unused.
     */

    public static PathByteArray emptyPath(Graph g) { 
        byte[] u  = new byte[ 0 ];
        byte[] uu = new byte[ g.size() ];
        for (byte i = 0; i < g.size(); i++ ){
            uu[i] = i;
        }
        return new PathByteArray(g, u, uu, Double.POSITIVE_INFINITY);
    }


    /**
     * standard path from point 0 to point n-1 to 0.
     */

    public static PathByteArray standardPath(Graph g) {
        byte[] u  = new byte[ g.size() ];
        byte[] uu = new byte[0];
        double c = 0.0;
        u[0] = (byte)0;
        for (byte i = 1; i < g.size(); i++ ){
            u[i] = i;
            c += cost( g.distance(i-1,i) );
        }
        // u[ g.size() ] = (byte)0;
        c += cost( g.distance(g.size()-1,0) ); //for circle
        PathByteArray p = new PathByteArray(g, u, uu, c);
        System.out.println("initial  = " + p);
        return p;
    }


    /**
     * get array of next possible paths.
     */

    public Path[] nextPaths() {
        PathByteArray[] pr = new PathByteArray[ unused.length ];
        for ( byte i = 0; i < unused.length; i++ ) {
            byte[] uu = new byte[ unused.length-1 ];
            byte m = 0;
            byte j = -1;
            for ( byte k = 0; k < unused.length; k++ ) {
                if ( k == i ) { 
                    j = unused[k];
                } else {
                    uu[ m ] = unused[k]; m++;
                }
            }
            // j != -1
            byte[] u = new byte[ used.length+1 ];
            System.arraycopy( used, 0, u, 0, used.length );
            u[ used.length ] = j;
            // compute new cost
            double c = 0.0;
            if ( cost < Double.POSITIVE_INFINITY ) {
               c = cost;
            }
            if ( length > 0 ) {
               int l = used[ length-1 ];
               int k = j;
               c += cost( graph.distance(l,k) );
            }
            // add circle cost
            if ( length+1 == maxsize ) {
                //System.out.print("#");
               int n = used[ 0 ];
               int k = j;
               c += cost( graph.distance(k,n) );
            }
            pr[i] = new PathByteArray( graph, u, uu, c ); 
        }
        return pr;
    }


    public String toString() {
        StringBuffer s = new StringBuffer();
        s.append("[ ");
        for ( int i = 0; i < length; i++ ) {
            s.append( used[i] );
            if ( i+1 < length ) {
               s.append(",");
            }
        }
        s.append(" | cost="+cost+" ]");
        return s.toString();
    }

    protected static double cost(double c) {
        if ( c >= 0.0 ) {
            return c;
        }
        double r = Math.log( c );
        for (int i = 0; i < 100; i++ ) {
            r *= c;
        }
        r = r/2.0;
        for (int i = 0; i < 100; i++ ) {
            r /= c;
        }
        r = r/2.0;
        return Math.exp( r );
    }

}