
/**
 * A sequential algorithm for an euclidean 2d TSP Problem
 */

public class SeqByteTSP implements TSPInf {

    protected Graph graph;
    protected Point[] points;
    protected double best;   // beware of threads
    protected Path bestPath; // beware of threads
    protected long iter;
    protected long maxiter;


    public SeqByteTSP(Point[] p) {
        this.points = p;
        graph = new PlaneGraph(p);
    }


    public Path getBest() {
        return getBest(Long.MAX_VALUE);
    }

    public Path getBest(long max) {
        iter = 0; 
        maxiter = max;
        bestPath = PathByteArray.standardPath(graph); // standard path
        best = bestPath.cost();
        if ( bestPath.length() <= 2 ) {
            return bestPath;
        }
        Path path = PathByteArray.emptyPath(graph); // empty path
        getBest(path);
        System.out.println("iterations = " + iter);
        return bestPath;
    }

    public void getBest(Path path) {
        Path[] ps = path.nextPaths(); 
        iter += ps.length;
        if ( path.length()+1 == path.maxsize() ) {
           for ( int i = 0; i < ps.length; i++ ) { // branch
               //System.out.println("seq path = " + ps[i]);
               if ( ps[i].cost() < best ) {
                  best = ps[i].cost();
                  bestPath = ps[i];
                  System.out.println("seq best = " + bestPath);
               }
           }
           return;
        }
        if ( iter >= maxiter ) {
            return;
        }
        for ( int i = 0; i < ps.length; i++ ) { // branch
            //System.out.println("seq path part = " + ps[i]);
            if ( ps[i].cost() < best ) { // cut
               getBest( ps[i] ); // changes best and bestPath
            }
        }
        return;
    }


}