/*
 * $Id: Semaphore.java,v 1.7 2004/01/17 12:19:18 kredel Exp $
 */

//package edu.unima.ky.parallel;
package thread;

import util.Logger;

import java.io.Serializable;
//import org.apache.log4j.Logger;

/**
 * Semaphore.
 * This class is safe against Thread InterruptedException.
 * @author Akitoshi Yoshida
 * @author Heinz Kredel.
 */
public class Semaphore implements Serializable {
  private int init;
  private int s;
  private int del;

  //private static Logger logger = Logger.getLogger("edu.jas");
   private static Logger logger = new Logger();

/**
 * Constructs a default semaphore.
 */
  public Semaphore() {
    this(0);
  }

/**
 * Constructs a semaphore with the given upper limit value.
 * @param i initial limit.
 */
public Semaphore(int i) {
    if (i >=0) { init = i; } else { init = 0; }
    s = init;
    del = 0;
  }

/**
 * Finalizes this object.
 */
  protected void finalize() throws Throwable {
    if (init != s) { 
        int x = s - init;
        logger.warn("Semaphore: " + x + " pending operations."); 
    }
   super.finalize();
  }

/**
 * Performs the P operation.
 * @throws InterruptedException.
 */
public synchronized void P() throws InterruptedException {
    while (s <= 0) {
      del++;
      try { this.wait(); 
      } finally { del--; }
    }
    s--;
  }

/**
 * Performs the time limited P operation.
 * @param m time to wait.
 * @return true if P succeeded, else false.
 * @throws InterruptedException.
 */
public synchronized boolean P(int m) throws InterruptedException {
    if (s <= 0) {
      del++;
      try { this.wait(m); 
      } finally { del--; }
      if (s <= 0) return false;
    }
    s--;
    return true;
  }

/**
 * Performs the V operation.
 */
  public synchronized void V() {
    s++;
    if (del > 0) {
      this.notify();
    }
  }

/**
 * Checks if Semaphore is positive.
 * @return true if Semaphore is positive, else false.
 */
public boolean isPositive() {
      return (s > 0);
  }

}
